//Функция генерации обучающих данных (для динамики выходных характеристик)
function [SystemOutputCharacteristics,..//Выходные характеристики
          SystemParametrsIndexes,..//Индексы параметров системы, для которых рассчитывались выходные характеристики
          SystemParametrs,..//Параметры системы, для которых рассчитывались выходные характеристики
          TimesMomemts..//Моменты времени
          ]=AdvancedModelLearningDataTimeGrid(SystemOutputCharacteristicsNames,...//Имена выходных характеристик системы
                                              SystemOutputCharacteristicsNamesIndexes,...//Индексы выходных характеристик системы
                                              SystemOutputCharacteristicsIndexes,...//Индексы выходных характеристик системы (в именах)
                                              SystemShemaName,...//Имя схемы системы
                                              TimeMoments,...//Моменты времени
                                              SystemParametrsNames,...//Имена параметров, определяющих динамику
                                              SystemParametrsValues,...//Значения параметров, определяющих динамику
                                              absolutTolErr,..//Максимальное значение абсолютной ошибки
                                              relativeTolErr,..//Минимальное значение относительной ошибки
                                              simulationTime,..//Время симуляции
                                              timeStepMax,..//Максимальный шаг интергирования по времени
                                              Solver,..//Решатель
                                              toleranceTime..//Максимальная ошибка времени
                                              )
    //Проверяем число аргументов
    [nOut,nArg] = argn();//Число аргументов
    if nArg < 7 then
        error('Function AdvancedModelLearningDataTimeGrid takes 7 or more arguments (maximum 13 arguments)');
    end
    
    //Восполняем недостающие аргументы
    select nArg
    case 7 then
        absolutTolErr = [];
        relativeTolErr = [];
        simulationTime = [];
        timeStepMax = [];
        Solver = [];
        toleranceTime = [];
    case 8 then
        relativeTolErr = [];
        simulationTime = [];
        timeStepMax = [];
        Solver = [];
        toleranceTime = [];
    case 9 then
        simulationTime = [];
        timeStepMax = [];
        Solver = [];
        toleranceTime = [];
    case 10 then
        timeStepMax = [];
        Solver = [];
        toleranceTime = [];
    case 11 then
        Solver = [];
        toleranceTime = [];
    case 12 then
        toleranceTime = [];
    end
           
    //Приволим аргументы к виду матрица - строка
    SystemOutputCharacteristicsNamesIndexes = matrix(SystemOutputCharacteristicsNamesIndexes,1,-1);
    SystemOutputCharacteristicsIndexes = matrix(SystemOutputCharacteristicsIndexes, 1, -1);
    TimeMoments = matrix(TimeMoments, 1, -1);
        
    //Проверяем корректность аргументов
    if type(SystemOutputCharacteristicsNamesIndexes) ~= 1 then
        error('The second argument of function AdvancedModelLearningDataTimeGrid is matrix of integer');
    end
    if type(SystemOutputCharacteristicsIndexes) ~= 1 then
        error('The third argument of function AdvancedModelLearningDataTimeGrid is matrix of integer');
    end
    if size(SystemOutputCharacteristicsNamesIndexes,'*') ~= size(SystemOutputCharacteristicsIndexes,'*') then
        error('The sizes of arguments SystemOutputCharacteristicsNamesIndexes and SystemOutputCharacteristicsIndexes of function AdvancedModelLearningDataTimeGrid must be equality');
    end
    if (type(SystemShemaName) ~= 10) ||..
       (size(SystemShemaName, '*') > 1) then
        error('The 4th argument of function AdvancedModelLearningDataTimeGrid is scalar string');
    end
    if type(TimeMoments) == 17 then
        for ind=1:size(TimeMoments,'*')
            if type(TimeMoments{ind}) ~= 1 then
                error('The 5th argument of function AdvancedModelLearningDataTimeGrid is cell-array of functions');
            end
        end
    else
        error('The 5th argument of function AdvancedModelLearningData is cell-array of functions');
    end
    nCurves = size(SystemParametrsValues)(1);//Число кривых
    if size(TimeMoments,'*') ~= nCurves then
        error('The number of times moments must be equal to the number of possible dynamics');
    end
    
    //Загружаем схему системы
    SystemShema = xcosDiagramToScilab(SystemShemaName);
    
    //Вычислительный эксперимент
    SystemOutputCharacteristics = [];
    if nOut >= 2 then
        SystemParametrsIndexes = [];
    end
    if nOut >= 4 then
        TimesMomemts = [];
    end
    for ind=1:nCurves
        //Проводим вычислительный эксперимент
        systemOutputCharacteristics = ComputingExperiment(SystemOutputCharacteristicsNames,...//Имена выходных характеристик системы
                                                          SystemShema,..//Схема системы
                                                          SystemParametrsNames,..//Имена параметров
                                                          SystemParametrsValues(ind,:),..//Значения параметров
                                                          absolutTolErr(ind),..//Максимальное значение абсолютной ошибки
                                                          relativeTolErr(ind),..//Минимальное значение относительной ошибки
                                                          simulationTime(ind),..//Время симуляции
                                                          timeStepMax(ind),..//Максимальный шаг интергирования по времени
                                                          Solver(ind),..//Решатель
                                                          toleranceTime(ind)..//Максимальная ошибка времени
                                                          );
                                                          
        //Приводим выходные характеристики системы к одному временному масштабу
        [DynamicsInTimeGrid,..
         mTimesMoments] = DynamicsToTimeGrid(systemOutputCharacteristics,..//Массив динамик
                                             SystemOutputCharacteristicsNamesIndexes,..//Индексы имен динамик
                                             SystemOutputCharacteristicsIndexes,..//Индексы динамик (в именах)
                                             TimeMoments{ind}..//Моменты времени
                                             );
                                                              
        //Приводим параметры и формируем результат
        SystemOutputCharacteristics = [SystemOutputCharacteristics;..
                                       DynamicsInTimeGrid];    
        if nOut >= 2 then
            SystemParametrsIndexes = [SystemParametrsIndexes;..
                                      repmat(ind,..
                                             [1,size(DynamicsInTimeGrid)(1)])'];
        end
        if nOut >= 4 then
            TimesMomemts = [TimesMomemts;mTimesMoments];
        end
    end
    
    //Формируем матрицу выходных параметров системы    
    if nOut >= 3 then
        SystemParametrs = SystemParametrsValues(SystemParametrsIndexes,:);//Формируем матрицу параметров системы
    end
endfunction

//Приведение динамик к единой заданной пользователем временной сетке
function [DynamicsInTimeGrid,..//Динамики в единой временной сетке
          TimesMoments..//Моменты времени
          ]=DynamicsToTimeGrid(Dynamics,..//Массив динамик
                               DynamicsNamesIndexes,..//Индексы имен динамик
                               DynamicsIndexes,..//Индексы динамик (в именах)
                               TimesMoments..//Моменты времени
                               )
    //Проверяем корректность аргументов
    if isempty(Dynamics) || isempty(DynamicsNamesIndexes) ||..
       isempty(DynamicsIndexes)  then
        DynamicsInTimeGrid = [];
        return;
    end
    
    //Приводим моменты времени к вектору-столбцу
    TimesMoments = matrix(TimesMoments,-1,1);
    
    //Выполняем выборку
    DynamicsInTimeGrid = [];
    for ind=1:size(DynamicsIndexes,'*')//Вспомогательная функция фильтрации по индексу
        dynamicAfterConvercion = interp1(Dynamics(DynamicsNamesIndexes(ind)).time,...
                                         Dynamics(DynamicsNamesIndexes(ind)).values(:,DynamicsIndexes(ind)),...
                                         TimesMoments, 'spline', 'extrap');
        DynamicsInTimeGrid = [DynamicsInTimeGrid,..
                              dynamicAfterConvercion];
    end
endfunction
